@@ -10,14 +10,17 @@ __all__ = ('InputStream', )
 class InputStream(object):
     """
     Parser for VT100 input stream.
-    Feed the data through the ``feed`` method and the correct callbacks of the
-    attached interfaces will be called.
+
+    Feed the data through the `feed` method and the correct callbacks of the
+    `inputstream_handler` will be called.
 
     ::
 
-        h = CommandLineHandler()
+        h = InputStreamHandler()
         i = InputStream(h)
         i.feed('data\x01...')
+
+    :attr handler: :class:`~prompt_toolkit.inputstream_handler.InputStreamHandler` instance.
     """
     # Lookup table of ANSI escape sequences for a VT100 terminal
     CALLBACKS = {
@@ -47,6 +50,7 @@ class InputStream(object):
         '\x18': 'ctrl_x', # Control-X
         '\x19': 'ctrl_y', # Control-Y (25)
         '\x1a': 'ctrl_z', # Control-Z
+        '\x1f': 'ctrl_underscore', # Control-underscore
         '\x7f': 'backspace', # (127) Backspace
            ### '\x1b': 'escape',
         '\x1b[A': 'arrow_up',
@@ -62,6 +66,7 @@ class InputStream(object):
         '\x1b[6~': 'page_down',
         '\x1b[7~': 'home', # xrvt
         '\x1b[8~': 'end', # xrvt
+        '\x1b[Z': 'backtab', # shift + tab
 
         '\x1bOP': 'F1',
         '\x1bOQ': 'F2',
@@ -73,6 +78,16 @@ class InputStream(object):
         '\x1b[19~': 'F8',
         '\x1b[20~': 'F9',
         '\x1b[21~': 'F10',
+        '\x1b[23~': 'F11',
+        '\x1b[24~': 'F12',
+        '\x1b[25~': 'F13',
+        '\x1b[26~': 'F14',
+        '\x1b[28~': 'F15',
+        '\x1b[29~': 'F16',
+        '\x1b[31~': 'F17',
+        '\x1b[32~': 'F18',
+        '\x1b[33~': 'F19',
+        '\x1b[34~': 'F20',
     }
 
     def __init__(self, handler, stdout=None):
@@ -92,7 +107,7 @@ class InputStream(object):
         """
         State machine for the input parser.
         """
-        shift_buffer = ''
+        shift_buffer = '' # TODO: rename to 'buffer'
 
         while True:
             options = self.CALLBACKS
@@ -142,6 +157,7 @@ class InputStream(object):
         Feed the input stream.
         """
         assert isinstance(data, six.text_type)
+
         #print(repr(data))
 
         for c in data:
