@@ -79,7 +79,9 @@ class TerminalCodes:
 
 
 class Char(object):
-    def __init__(self, char=' ', style=None): # TODO: add __slots__
+    __slots__ = ('char', 'style')
+
+    def __init__(self, char=' ', style=None):
         self.char = char
         self.style = style # TODO: maybe we should still use `token` instead of
                            #       `style` and use the actual style in the last step of the renderer.
@@ -105,7 +107,10 @@ class Char(object):
 
     @property
     def width(self):
-        return wcwidth(self.char)
+        # We use the `max(0, ...` because some non printable control
+        # characters, like e.g. Ctrl-underscore get a -1 wcwidth value.
+        # It can be possible that these characters end up in the input text.
+        return max(0, wcwidth(self.char))
 
 
 class Screen(object):
@@ -175,14 +180,7 @@ class Screen(object):
 
         # Insertion of a 'visible' character.
         else:
-            # Get style
-            try:
-                style = self._style.style_for_token(token)
-            except KeyError:
-                style = None
-
-            # Add char to buffer
-            self._buffer[self._y][self._x] = Char(char=char, style=style)
+            self.write_at_pos(self._y, self._x, char, token)
 
             # Move cursor position
             if is_input:
@@ -194,6 +192,31 @@ class Screen(object):
             else:
                 self._x += char_width
 
+    def write_at_pos(self, y, x, char, token):
+        """
+        Write character at position (y, x).
+        (Truncate when character is outside margin.)
+        """
+        # Get style
+        try:
+            style = self._style.style_for_token(token)
+        except KeyError:
+            style = None
+
+        # Add char to buffer
+        if y < self._columns:
+            self._buffer[y][x] = Char(char=char, style=style)
+
+    def write_highlighted_at_pos(self, y, x, data):
+        """
+        Write (Token, text) tuples at position (y, x).
+        (Truncate when character is outside margin.)
+        """
+        for token, text in data:
+            for c in text:
+                self.write_at_pos(y, x, c, token)
+                x += wcwidth(c)
+
     def write_highlighted(self, data, is_input=True):
         """
         Write (Token, text) tuples to the screen.
@@ -230,9 +253,13 @@ class Screen(object):
                             }
 
     def output(self):
+        """
+        Return (string, last_y, last_x) tuple.
+        """
         result = []
 
         rows = max(self._buffer.keys()) + 1
+        c = 0
 
         for y, r in enumerate(range(0, rows)):
             line_data = self._buffer[r]
@@ -242,16 +269,92 @@ class Screen(object):
                 c = 0
                 while c < cols:
                     result.append(line_data[c].output())
-                    c += line_data[c].width
+                    c += (line_data[c].width or 1)
 
             if y != rows - 1:
                 result.append(TerminalCodes.CRLF)
 
-        return ''.join(result)
+        return ''.join(result), y, c
+
+
+class _CompletionMenu(object):
+    """
+    Helper for drawing the complete menu to the screen.
+    """
+    def __init__(self, screen, complete_state, max_height=7):
+        self.screen = screen
+        self.complete_state = complete_state
+        self.max_height = max_height
+
+    def _get_origin(self):
+        """
+        Return the position of the menu.
+        We calculate this by mapping the cursor position (from the
+        complete_state) in the _cursor_mappings of the screen object.
+        """
+        return self.screen._cursor_mappings[
+                (self.complete_state.original_document.cursor_position_row,
+                 self.complete_state.original_document.cursor_position_col)]
+
+    def write(self):
+        """
+        Write the menu to the screen object.
+        """
+        completions = self.complete_state.current_completions
+        index = self.complete_state.complete_index
+
+        # Get position of the menu.
+        y, x = self._get_origin()
+        y += 1
+        x = max(0, x - 1)
+
+        # Calculate width of completions menu.
+        menu_width = max(len(c.display) for c in self.complete_state.current_completions)
+
+        # Decide which slice of completions to show.
+        if len(completions) > self.max_height and index > self.max_height / 2:
+            slice_from = min(
+                        index - self.max_height / 2, # In the middle.
+                        len(completions) - self.max_height # At the bottom.
+                    )
+        else:
+            slice_from = 0
+
+        slice_to = min(slice_from + self.max_height, len(completions))
+
+        # Create a function which decides at which positions the scroll button should be shown.
+        def is_scroll_button(row):
+            items_per_row = float(len(completions)) / min(len(completions), self.max_height)
+            items_on_this_row_from = row * items_per_row
+            items_on_this_row_to = (row + 1) * items_per_row
+            return items_on_this_row_from <= self.complete_state.complete_index < items_on_this_row_to
+
+        # Write completions to screen.
+        for i, c in enumerate(completions[slice_from:slice_to]):
+            if i + slice_from == index:
+                token = Token.CompletionMenu.CurrentCompletion
+            else:
+                token = Token.CompletionMenu.Completion
+
+            if is_scroll_button(i):
+                button = (Token.CompletionMenu.ProgressButton, ' ')
+            else:
+                button = (Token.CompletionMenu.ProgressBar, ' ')
+
+            self.screen.write_highlighted_at_pos(y+i, x, [
+                        (Token, ' '),
+                        (token, ' %%-%is ' % menu_width % c.display),
+                        button,
+                        (Token, ' '),
+                        ])
 
 
 class Renderer(object):
     highlight_current_line = False
+
+    #: Boolean to indicate whether or not the completion menu should be shown.
+    show_complete_menu = True
+
     screen_cls = Screen
 
     def __init__(self, stdout=None, style=None):
@@ -259,7 +362,6 @@ class Renderer(object):
         self._style = style or DefaultStyle
 
         # Reset position
-        self._lines_in_use = 0
         self._cursor_line = 0
 
     def get_width(self):
@@ -290,7 +392,7 @@ class Renderer(object):
 
         # Highlight current line.
         if self.highlight_current_line and not (render_context.accept or render_context.abort):
-            screen.highlight_line(render_context.code_obj.document.cursor_position_row - 1)
+            screen.highlight_line(render_context.code_obj.document.cursor_position_row)
 
         # Highlight regions
         if render_context.highlight_regions:
@@ -298,6 +400,10 @@ class Renderer(object):
                 for i in range(start_column, end_column):
                     screen.highlight_character(start_row-1, i, bgcolor='444444', color='eeeeee')
 
+        # Write completion menu.
+        if self.show_complete_menu and render_context.complete_state:
+            _CompletionMenu(screen, render_context.complete_state).write()
+
         return screen
 
     def _render_to_str(self, render_context):
@@ -314,26 +420,25 @@ class Renderer(object):
 
         # Generate the output of the new screen.
         screen = self._get_new_screen(render_context)
-        write(screen.output())
+        o, last_y, last_x = screen.output()
+        write(o)
 
         # Move cursor to correct position.
         if render_context.accept or render_context.abort:
-            self._lines_in_use = 0
             self._cursor_line = 0
             write(TerminalCodes.CRLF)
         else:
             cursor_y, cursor_x = screen._cursor_mappings[
-                            render_context.code_obj.document.cursor_position_row - 1,
+                            render_context.code_obj.document.cursor_position_row,
                             render_context.code_obj.document.cursor_position_col]
 
-            if screen._y - cursor_y:
-                write(TerminalCodes.CURSOR_UP(screen._y - cursor_y))
-            if screen._x > cursor_x:
-                write(TerminalCodes.CURSOR_BACKWARD(screen._x - cursor_x))
-            if screen._x < cursor_x:
-                write(TerminalCodes.CURSOR_FORWARD(cursor_x - screen._x))
+            if last_y - cursor_y:
+                write(TerminalCodes.CURSOR_UP(last_y - cursor_y))
+            if last_x > cursor_x:
+                write(TerminalCodes.CURSOR_BACKWARD(last_x - cursor_x))
+            if last_x < cursor_x:
+                write(TerminalCodes.CURSOR_FORWARD(cursor_x - last_x))
 
-            self._lines_in_use = screen._y
             self._cursor_line = cursor_y
 
         return ''.join(output)
@@ -348,6 +453,9 @@ class Renderer(object):
         for line in self._in_columns([ c.display for c in completions ]):
             self._stdout.write('%s\r\n' % line)
 
+        # Reset position
+        self._cursor_line = 0
+
         return
         if many: # TODO: Implement paging
             'Display all %i possibilities? (y on n)'
